import {
  Class,
} from '@overflow/commons/core/reflect';
import { ClassType } from '@overflow/commons/core/type';
import { ApplicationContext } from './context';
import {
  InjectableClass,
  InjectableFactory,
  InjectableValue,
  WebApplicationAnnotation,
} from './decorators';

export class Application {
  private primaryClass: ClassType;
  private applicationContext: ApplicationContext;

  public static run(primaryClass: ClassType): void {
    let app: Application = new Application(primaryClass);
  }

  public constructor(primaryClass: ClassType) {
    this.primaryClass = primaryClass;

    this.createContext();
  }

  private createContext(): void {
    let clazz: Class = Class.forClass(this.primaryClass);

    let wa: WebApplicationAnnotation = clazz.getOwnAnnotation(WebApplicationAnnotation);
    if (undefined === wa) {
      console.warn(`Class is not WebApplication type. add @WebApplication annotation to class[${this.primaryClass.name}]`);
      return;
    }

    let context: ApplicationContext = new ApplicationContext();
    // context.registerInjectables(wa.attributes.injectables);

    wa.attributes.injectables.forEach(injectable => {
      if (undefined !== (<InjectableClass>injectable).injectableType) {
        context.registerInjectable()
      } else if (undefined !== (<InjectableFactory>injectable).injectableFactory) {
        console.log(`InjectableFactory`);
      } else if (undefined !== (<InjectableValue>injectable).injectableValue) {
        console.log(`InjectableValue`);
      } else if (typeof injectable === 'function') {
        console.log(`Class`);
      }
    });

  }
}
