import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  ClassType,
  PropertyKeyType,
} from '@overflow/commons/core/type';

export interface InjectableClass {
  injectType: ClassType;
  injectableType: ClassType;
}

export interface InjectableFactory {
  injectType: ClassType;
  injectableFactory: (...params: any[]) => any;
}

export interface InjectableValue {
  injectName: PropertyKeyType;
  injectableValue: any;
}

export type InjectablesType = (ClassType | InjectableClass | InjectableFactory | InjectableValue)[];

export interface WebApplicationAnnotationAttributes {
  injectables?: InjectablesType;
  configurations?: ClassType[];
}

export class WebApplicationAnnotation implements Annotation {
  public readonly attributes: WebApplicationAnnotationAttributes;

  public constructor(attributes: WebApplicationAnnotationAttributes) {
    this.attributes = attributes;
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    // no op
  }

}

export const WebApplication = Decorator.create(WebApplicationAnnotation);
