import {
  Annotation,
  Decorator,
} from '@overflow/commons/core/reflect';

import {
  PropertyKeyType,
  TypeUtil,
} from '@overflow/commons/core/type';


export interface InjectAnnotationAttributes {
  name?: string[];
}

export class InjectAnnotation implements Annotation {
  public readonly attributes: InjectAnnotationAttributes;

  public constructor(name: string | string[] | InjectAnnotationAttributes) {

    if (undefined === name) {
      this.attributes = {
      };
      return;
    }

    if (TypeUtil.isString(name)) {
      this.attributes = {
        name: [<string>name],
      };
    } else if (TypeUtil.isArray(name)) {
      this.attributes = {
        name: <string[]>name,
      };
    } else {
      this.attributes = <InjectAnnotationAttributes>name;
    }
  }

  public classDecorator<TFunction extends Function>(target: TFunction): TFunction | void {
    console.log('Inject');
  }

  public propertyDecorator?(target: Object, propertyKey: PropertyKeyType): void {
    console.log('Inject');
  }

  public methodDecorator<T>(target: Object, propertyKey: PropertyKeyType,
                            descriptor: TypedPropertyDescriptor<T>): TypedPropertyDescriptor<T> | void {
    console.log('Inject');
  }

  public parameterDecorator?(target: Object, propertyKey: PropertyKeyType, parameterIndex: number): void {
    console.log('Inject');
  }

}

export const Inject = Decorator.create(InjectAnnotation);
